import pygame
from pygame.locals import *
from sys import exit
import random
import os

cwd = os.getcwd()

class Player(pygame.sprite.Sprite):

    def __init__(self, imageList, speed, accel, hp, att, defence, team, x, y):
        self.imageList = imageList

        if team == -1:
            imagelistcopy, self.imageList = self.imageList, []
            for imageSublist in imagelistcopy:
                sublist = []
                for image in imageSublist:
                    sublist += [pygame.transform.flip(image, True, False)]
                self.imageList += [sublist]
            
        self.x = x
        self.y = y

        self.dx = 0

        self.att = att
        self.hp = hp
        self.defence = defence
        self.speed = speed
        self.accel = accel
        
        self.imageNum1 = 0
        self.imageNum2 = 0
        
        self.currentImg = self.imageList[0][0]
        
        left = x - self.currentImg.get_width()/2
        top = y - self.currentImg.get_width()
        self.rect = pygame.Rect(left, top, 
                                self.currentImg.get_width(),
                                self.currentImg.get_height())
        
        self.wo=self.currentImg.get_width()/2 #WidthOffset, WO.
        self.ho=self.currentImg.get_height()/2
        
        self.team = team
        self.type = 'player'

        self.counter = 1

    def update(self, dead = False):
               
        self.counter += 1
        
        if dead == False:
        
            self.dx += self.accel
            
            if self.dx > self.speed:
                self.dx = self.speed
                
            self.x += self.dx*self.team
            
            self.rect.left = self.x - self.currentImg.get_width()/2
            self.rect.top = self.y - self.currentImg.get_height()

            if self.x <100 + self.wo:
                self.x = 100 + self.wo
                self.dx = 0

            if self.x >900 - self.wo:
                self.x = 900 - self.wo
                self.dx = 0

        if self.counter == 10:
            self.counter = 1

            self.swapImg(self.imageNum1, (self.imageNum2+1)%len(self.imageList[self.imageNum1]))

        

    def checkCollide(self, other):
        if abs(self.x - other.x) < self.wo + other.wo:
            return True
        return False

    def draw(self, screen):
        screen.blit(self.currentImg,self.rect)        

    def swapImg(self, a, b):
        self.imageNum1 = a
        self.imageNum2 = b
        
        self.currentImg = self.imageList[self.imageNum1][self.imageNum2]

        self.rect.left = self.x - self.currentImg.get_width()/2
        self.rect.top = self.y - self.currentImg.get_height()

        self.wo=self.currentImg.get_width()/2 #WidthOffset, WO.
        self.ho=self.currentImg.get_height()/2

class MeleeCombatant(Player):

    def __init__(self, imageList, speed, accel, hp, att, defence, team, x, y):
        Player.__init__(self, imageList, speed, accel, hp, att, defence, team, x, y)
        self.diecounter = len(self.imageList[1]) * 10 - 10

    def update(self, spritelist):
        if self.hp <= 0:
            if self.diecounter == len(self.imageList[1]) * 10 - 10:
                self.swapImg(1,0)
            self.diecounter -= 1
            if self.diecounter <= 0:
                spritelist.remove(self)
            Player.update(self, True)
        else:
            for sprite in spritelist:
                if self.checkCollide(sprite) and sprite.type == 'player':
                    if self.team != sprite.team:
                        self.attack(sprite)
            Player.update(self)

    def attack(self, other):
        if self.att > other.defence:
            other.hp -= self.att - other.defence
            other.dx = -2*(self.att - other.defence) - 2
        else:
            other.dx = -2

class Zombie(MeleeCombatant):

    def __init__(self, team, x, y = 300):
        MeleeCombatant.__init__(self, compiledZombieList, 1, .1, 10, 1, 0, team, x,y)

class BG(pygame.sprite.Sprite):
    
    def __init__(self, image):
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.rect = pygame.Rect(0, 0, self.image.get_width(), self.image.get_height())

    def draw(self, screen):
        screen.blit(self.image,self.rect)

class Button(pygame.sprite.Sprite):
    
    def __init__(self, image, x, y):
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.rect = pygame.Rect(x, y, self.image.get_width(), self.image.get_height()) #making rect

    def draw(self, screen):
        screen.blit(self.image,self.rect)

    def clicked(self,x,y):
        return self.rect.collidepoint(x,y)

def makeImage(image,transparantBG = True,x=1,y=1):
    image = os.path.join(cwd,'data', image)
    image = pygame.image.load(image)
    image = image.convert()
    if transparantBG:
        image.set_colorkey(image.get_at((x,y)))
    return image

def menu(screen, clock, title):
    title = BG(title)
    go = Button(makeImage('next.png', False), 900, 450)
    while True:
        timePassed = clock.tick(30)
        timePassed /= 1000.0
        title.draw(screen)
        go.draw(screen)
        pygame.display.update()
        for event in pygame.event.get():
            if event.type == QUIT:
                quit() 

            elif event.type == MOUSEBUTTONDOWN:
                x,y = pygame.mouse.get_pos()
                if go.clicked(x,y):
                    return None


def charChose(screen, side, creditsAvailable, zombieList = []):

    font = pygame.font.Font(None, 25)
    
    bg = BG(makeImage('labBG.png', False))
    
    keylist = []

    selectUnitBack = Button(makeImage('back.png', False), 350, 175)
    selectUnitNext = Button(makeImage('next.png', False), 550, 175)

    speedMore = Button(makeImage('more.png', False),215,100)
    speedLess = Button(makeImage('less.png', False),50,100)

    healthMore = Button(makeImage('more.png', False),215,300)
    healthLess = Button(makeImage('less.png', False),50,300)

    defenceMore = Button(makeImage('more.png', False),850,300)
    defenceLess = Button(makeImage('less.png', False),700,300)

    attackMore = Button(makeImage('more.png', False),850,100)
    attackLess = Button(makeImage('less.png', False),700,100)

    procede = Button(makeImage('procede.png', False), 400, 450)
    newZombie = Button(makeImage('newzombie.png', False), 500, 450)

    buttons = [speedMore,speedLess,healthMore,healthLess,defenceMore,defenceLess,attackMore,attackLess,procede,newZombie]

    zombie = Zombie(0, 500, 200)

    cost = 40
    
    info = [font.render('Zombie:', True, (0,0,0)),font.render('Speed: {}'.format(zombie.speed), True, (0,0,0)),font.render('Health: {}'.format(zombie.hp), True, (0,0,0)),font.render('Attack: {}'.format(zombie.att), True, (0,0,0)),font.render('Defence: {}'.format(zombie.defence), True, (0,0,0)),font.render('Credits: {}'.format(creditsAvailable), True, (0,0,0)),font.render('Cost: {}'.format(cost), True, (0,0,0))]

    warning = font.render('', True, (0,0,0))

    while True:
        for event in pygame.event.get():
            if event.type == QUIT:
                exit()
            elif event.type == KEYDOWN:
                keyList += [event.key]
            elif event.type == KEYUP:
                if event.key in keyList:
                    keyList.remove(event.key)
            elif event.type == MOUSEBUTTONDOWN:
                x,y = pygame.mouse.get_pos()
                if speedMore.clicked(x,y):
                    zombie.speed += 1
                    cost += 10
                elif speedLess.clicked(x,y):
                    if zombie.speed > 0:
                        zombie.speed -= 1
                        cost -= 10
                elif healthMore.clicked(x,y):
                    if zombie.speed > 0:
                        zombie.hp += 1
                        cost += 4
                elif healthLess.clicked(x,y):
                    if zombie.hp > 0:
                        zombie.hp -= 1
                        cost -= 4
                elif defenceMore.clicked(x,y):
                        zombie.defence += 1
                        cost += 20
                elif defenceLess.clicked(x,y):
                    if zombie.defence > 0:
                        zombie.defence -= 1
                        cost -= 20
                elif attackMore.clicked(x,y):
                    zombie.att += 1
                    cost += 20
                elif attackLess.clicked(x,y):
                    if zombie.att > 0:
                        zombie.att -= 1
                        cost -= 20
                elif procede.clicked(x,y):
                    if cost <= creditsAvailable:
                        z = Zombie(side, 500 - random.randint(200,400)*side)
                        z.att = zombie.att
                        z.defence = zombie.defence
                        z.speed = zombie.speed
                        z.hp = zombie.hp
                        zombieList += [z]
                        return zombieList
                    else:
                        warning = font.render('Not enough money.', True, (0,0,0))
                elif newZombie.clicked(x,y):
                    if cost + 40 <= creditsAvailable:
                        z = Zombie(side, 500 - random.randint(200,400)*side)
                        z.att = zombie.att
                        z.defence = zombie.defence
                        z.speed = zombie.speed
                        z.hp= zombie.hp
                        zombieList += [z]
                        return charChose(screen, side, creditsAvailable - cost,zombieList)
                    else:
                        warning = font.render('Not enough money.', True, (0,0,0))
                info = [font.render('Zombie:', True, (0,0,0)),font.render('Speed: {}'.format(zombie.speed), True, (0,0,0)),font.render('Health: {}'.format(zombie.hp), True, (0,0,0)),font.render('Attack: {}'.format(zombie.att), True, (0,0,0)),font.render('Defence: {}'.format(zombie.defence), True, (0,0,0)),font.render('Credits: {}'.format(creditsAvailable), True, (0,0,0)),font.render('Cost: {}'.format(cost), True, (0,0,0))]
                        
        bg.draw(screen)

        zombie.draw(screen)

        i = 300
        
        for line in info:
            screen.blit(line, (400,i))
            i += 14
        screen.blit(warning, (400,420))
        
        for button in buttons:
            button.draw(screen)

        pygame.display.update()
    

def battle(screen, objList, bg):
    keylist = []
    while True:
        
        for event in pygame.event.get():
            if event.type == QUIT:
                exit()
            elif event.type == KEYDOWN:
                keyList += [event.key]
            elif event.type == KEYUP:
                if event.key in keyList:
                    keyList.remove(event.key)

        for obj in objList:
            obj.update(objList)

        clock.tick(48)

        bg.draw(screen)

        for obj in objList:
            obj.draw(screen)

        pygame.display.update()

        team1win = True
        team2win = True
        
        for z in objList:
            if not z.team == 1:
                team1win = False
                break
        for z in objList:
            if not z.team == -1:
                team2win = False
                break
        if team1win:
            objList = []
            return 1
        elif team2win:
            objList = []
            return 2
            

pygame.mixer.pre_init(44100, 16, 2, 4096)     
pygame.init()
screen = pygame.display.set_mode((1000, 500))
pygame.display.set_caption('Test')

clock = pygame.time.Clock()

keyList = []

zombieWalkingList = []

for i in range(0, 11):
    zombieWalkingList += [makeImage('zombie_walking_' + str(i) + '.png')]

zombieDyingList = []

for i in range(0, 8):
    zombieDyingList += [makeImage('zombie_dying_' + str(i) + '.png')]

cImg = makeImage('court.png', False)

compiledZombieList = [zombieWalkingList, zombieDyingList]

#imageList, shadowImg, speed, accel, hp, att, defence, team, x, y, sOffsetX, sOffsetY

bg = BG(cImg)

zombieList = []

menu(screen, clock, makeImage('title.png',False))
    
while True:

    zombieList = charChose(screen, 1, 1000, [])
    zombieList = charChose(screen, -1, 1000, zombieList)

    winner = battle(screen, zombieList , bg)

    menu(screen, clock, makeImage('victory'+str(winner)+'.png',False))

    zombieList = []
